'use server';
import 'server-only';
import { createServerClient } from '@/utils/supabase';
import type { Result } from './index';

export type GetStorageProps = {
  bucket: string;
  url: string;
  seconds?: number;
  transform?: {
    width?: number;
    height?: number;
    quality?: number;
    format?: 'origin';
    resize?: 'cover' | 'contain' | 'fill';
  };
  download?: boolean;
};

export type UploadStorageProps = {
  bucket: string;
  path: string;
  file: File;
  options?: {
    cacheControl?: string;
    upsert?: boolean;
    contentType?: string;
  };
};

export async function getSignedUrl({
  bucket,
  url,
  seconds = 3600,
  transform,
  download = false,
}: GetStorageProps): Promise<Result<string>> {
  try {
    const supabase = await createServerClient();
    const { data, error } = await supabase.storage
      .from(bucket)
      .createSignedUrl(url, seconds, { transform });

    if (error) throw error;
    if (!data?.signedUrl) throw new Error('No signed URL returned');

    // Safely add download parameter if needed
    if (download) {
      const urlObj = new URL(data.signedUrl);
      urlObj.searchParams.append('download', '');
      return { success: true, data: urlObj.toString() };
    }

    return { success: true, data: data.signedUrl };
  } catch (error) {
    console.error('Could not get signed URL for asset!', error);
    return {
      success: false,
      error:
        error instanceof Error
          ? error.message
          : 'Unknown error getting signed URL',
    };
  }
}

export async function getPublicUrl({
  bucket,
  url,
  transform = {},
  download = false,
}: GetStorageProps): Promise<Result<string>> {
  try {
    const supabase = await createServerClient();
    const { data } = supabase.storage
      .from(bucket)
      .getPublicUrl(url, { transform });

    if (!data?.publicUrl) throw new Error('No public URL returned');

    // Safely add download parameter if needed
    if (download) {
      const urlObj = new URL(data.publicUrl);
      urlObj.searchParams.append('download', '');
      return { success: true, data: urlObj.toString() };
    }

    return { success: true, data: data.publicUrl };
  } catch (error) {
    console.error('Could not get public URL for asset!', error);
    return {
      success: false,
      error:
        error instanceof Error
          ? error.message
          : 'Unknown error getting public URL',
    };
  }
}

export async function uploadFile({
  bucket,
  path,
  file,
  options = {},
}: UploadStorageProps): Promise<Result<string>> {
  try {
    const supabase = await createServerClient();
    const { data, error } = await supabase.storage
      .from(bucket)
      .upload(path, file, options);

    if (error) throw error;
    if (!data?.path) throw new Error('No path returned from upload');

    return { success: true, data: data.path };
  } catch (error) {
    console.error('Could not upload file!', error);
    return {
      success: false,
      error:
        error instanceof Error ? error.message : 'Unknown error uploading file',
    };
  }
}

// Add a helper to delete files
export async function deleteFile({
  bucket,
  path,
}: {
  bucket: string;
  path: string[];
}): Promise<Result<null>> {
  try {
    const supabase = await createServerClient();
    const { error } = await supabase.storage.from(bucket).remove(path);

    if (error) throw error;

    return { success: true, data: null };
  } catch (error) {
    console.error('Could not delete file!', error);
    return {
      success: false,
      error:
        error instanceof Error ? error.message : 'Unknown error deleting file',
    };
  }
}

// Add a helper to list files in a bucket
export async function listFiles({
  bucket,
  path = '',
  options = {},
}: {
  bucket: string;
  path?: string;
  options?: {
    limit?: number;
    offset?: number;
    sortBy?: { column: string; order: 'asc' | 'desc' };
  };
}): Promise<Result<Array<{ name: string; id: string; metadata: any }>>> {
  try {
    const supabase = await createServerClient();
    const { data, error } = await supabase.storage
      .from(bucket)
      .list(path, options);

    if (error) throw error;
    if (!data) throw new Error('No data returned from list operation');

    return { success: true, data };
  } catch (error) {
    console.error('Could not list files!', error);
    return {
      success: false,
      error:
        error instanceof Error ? error.message : 'Unknown error listing files',
    };
  }
}
